require("lib/lincoln")
require("enums")

Globals = {
  hasRanAlready = false,
  isWormanDone = false,
  isHaldricDone = false,
  hasCivilianWalkedAway = false,
  isCibleHit = false,
  isArcheryDone = false,
  victory = false,
  hasPoorWeepingOneRun = false,
  TutorialGetObjectFirst = true,
  EdwardZoneFirst = true,
  lowered_bridge = false,
}

---Events
function Initialize(seed)
  InitializeLincoln()

  SetDoorLockedPC(GetDoorScript(DungeonTower.Door.dungeonEntrance), true)
  SetDoorLockedPC(GetDoorScript(MiscDoor.garissonEntrance), true)

  DeactivateActors({
	GetActor("Worman"),
    GetActor("Edward"),
    GetActor("Arbaletrier_Bleu_23"),
    GetActor("Epee_Bleu_36")
  })

  LockAI(GetActor("Poor_Weeping_One"), false)

  DeactivateItems({
    GetItem("Purses_7"),
    GetItem("Money_9"),
    GetItem("Clovers_11"),
    GetItem("Arrows_3"),
    GetItem("Money_6"),
    GetItem("Purses_5"),
    GetItem("Purses_10")
  })

  local reinforcements = { -- initiallly the reinforce are outmap
    "Reinforce_off",
    "Reinforce_1",
    "Reinforce_2",
    "Reinforce_3",
    "Reinforce_4",
    "Reinforce_5",
    "Reinforce_6",
    "Reinforce_7",
    "Reinforce_8",
    "taxman",
    "Civilian_9",
	}
  for _, actor in pairs(reinforcements) do
    Deactivate(GetActor(actor))
    LockAI(GetActor(actor), false)
    --SetActorLocation(GetActor(actor), NoWhere())
  end
 
  SetPersistentProperty(GetRobin(), PersistentProperty.arrows, 0)
  SetPersistentProperty(GetRobin(), PersistentProperty.purses, 0)
   
end

function Timer(seconds)
  if seconds % 15 == 0 and not Globals.isArcheryDone then
    SendMessage(GetActor("Shooting_Archer"), 1)
  end  
  if seconds % 10 == 0 and not Globals.isArcheryDone then
	Combact_soldier_1()
	Combact_soldier_2()
  end 
  HandleFlirtingSwordsman()
  HandlePoorWeepingOneRun()
  HandleHaldricDone()
  HandleCivilianWalkedAway() 
  
end


function CheckVictoryCondition(seconds)
  if Globals.victory then
    ChooseVictoryDefeatText(0)
    return 1
  end
end

function PostInitialize()
  AddObjective(0, true)
  StartSequence()
  SequenceDisplayPopupText(0)
  Then()
  SequenceDisplayPopupText(1)
  Then()
  SequenceDisplayPopupText(2)
  Then()
  SequenceCall(function()
	AssignPatrol(GetActor("Soldier_48"), GetPatrol("Patrol_97"))
  end)
  Then()
  SequenceCall(function()
	AssignPatrol(GetActor("Soldier_51"), GetPatrol("Patrol_99"))
  end)
  Then()
  SequenceCall(function()
	AssignPatrol(GetActor("Soldier_49"), GetPatrol("Patrol_101"))
  end)
   Then()
  SequenceCall(function()
	AssignPatrol(GetActor("Soldier_52"), GetPatrol("Patrol_31"))
  end)
 --  Then()
 -- SequenceCall(function()
--	AssignPatrol(GetActor("Archer_Bleu_38"), GetPatrol("Patrol_102"))
 -- end)
  EndSequence() 
end

--Functions
function HandleFlirtingSwordsman()
  if IsActorOutOfAction(GetActor("Flirting_Swordsman")) and not Globals.hasRanAlready then
    AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_66"))
    AssignPatrol(GetActor("WomanCivilianPoor_2"), GetPatrol("Patrol_59"))
    SetPatrolShouldRun(GetActor("WomanCivilianPoor_2"), true)
    Globals.hasRanAlready = true
  end
end

function HandlePoorWeepingOneRun()
  if not Globals.hasPoorWeepingOneRun and
      IsActorOutOfAction(GetActor("Epee_Bleu_25")) and
      IsActorOutOfAction(GetActor("Epee_Bleu_26")) and
      IsActorOutOfAction(GetActor("Lancier_Bleu_27")) and
      IsActorOutOfAction(GetActor("Lancier_Bleu_28")) then
    SendMessage(GetActor("Poor_Weeping_One"), 1)
  end
end

function HandleWormanDone()
  if not Globals.isWormanDone and IsBonusItemPickedUp(GetItem("Money_6")) then
    Globals.isWormanDone = true
    CompleteObjective(3)
    UpdateScrollState(GetScroll("Scroll_10"), ScrollState.collected)
  end
end

function HandleHaldricDone()
  if not Globals.isHaldricDone and IsActorDead(GetActor("Haldric")) then
    Globals.isHaldricDone = true
    CompleteObjective(4)
    UpdateScrollState(GetScroll("Scroll_11"), ScrollState.collected)
  end
end

function HandleCivilianWalkedAway()
  if not Globals.hasCivilianWalkedAway and
      (IsActorOutOfAction(GetActor("Lancier_Bleu_19")) or
        IsActorOutOfAction(GetActor("Lancier_Bleu_20")) or
        IsActorOutOfAction(GetActor("Lancier_Bleu_21"))) then
    AssignPatrol(GetActor("WomanCivilianPoor_7"), GetPatrol("Patrol_58"))
    Globals.hasCivilianWalkedAway = true
  end
end

ArchersDone = function()
  if not Globals.isArcheryDone then
    CompleteObjective(5)
    Globals.isArcheryDone = true
    UpdateScrollState(GetScroll("Scroll_1"), ScrollState.persistent)
    AssignSoldiersToOfficer()
    AssignPatrolPaths()
    HandleOfficerState()
  end
end

function AssignSoldiersToOfficer()
  local subordinates = {
	  GetActor("Soldier_53"),
    GetActor("Soldier_54"),
	  GetActor("Soldier_57"),
    GetActor("Soldier_58"),
    GetActor("Archer_Bleu_12"),
    GetActor("Archer_Bleu_13"),
    GetActor("Shooting_Archer"),
    GetActor("Epee_Bleu_16"),
    GetActor("Epee_Bleu_17"),
    GetActor("Epee_Bleu_18")
  }

  for _, subordinate in pairs(subordinates) do
    AddAsSquadMember(GetActor("Archer_Officer"), subordinate)
  end
end

function AssignPatrolPaths()
  local patrolPaths = {
	  { actor = GetActor("Soldier_55"),    	     patrol = "Patrol_113" },
    { actor = GetActor("Soldier_56"),     	   patrol = "Patrol_114" },
    { actor = GetActor("Lancier_Bleu_19"),     patrol = "Patrol_51" },
    { actor = GetActor("Lancier_Bleu_20"),     patrol = "Patrol_52" },
    { actor = GetActor("Lancier_Bleu_21"),     patrol = "Patrol_53" },
    { actor = GetActor("Hallebardier_Bleu_6"), patrol = "Patrol_3" }
  }

  for _, path in pairs(patrolPaths) do
    AssignPatrol(path.actor, GetPatrol(path.patrol))
  end
end

function HandleOfficerState()
  if not IsActorOutOfAction(GetActor("Archer_Officer")) then
    DisplayPopupText(6)
    AssignPatrol(GetActor("Archer_Officer"), GetPatrol("Patrol_39"))
  else
    AssignSecondaryPaths()
  end
end

function AssignSecondaryPaths()
  local secondaryPaths = {
	  { actor = GetActor("Soldier_55"),       patrol = "Patrol_113" },
    { actor = GetActor("Soldier_56"),       patrol = "Patrol_114" },
	  { actor = GetActor("Soldier_53"),       patrol = "Patrol_116" },
    { actor = GetActor("Soldier_54"),       patrol = "Patrol_115" },
  
    { actor = GetActor("Archer_Bleu_12"),   patrol = "Patrol_72" },
    { actor = GetActor("Archer_Bleu_13"),   patrol = "Patrol_71" },
    { actor = GetActor("Shooting_Archer"),  patrol = "Patrol_70" },
    { actor = GetActor("Epee_Bleu_16"),     patrol = "Patrol_69" },
    { actor = GetActor("Epee_Bleu_17"),     patrol = "Patrol_67" },
    { actor = GetActor("Epee_Bleu_18"),     patrol = "Patrol_68" }
  }

  for _, path in pairs(secondaryPaths) do
    AssignPatrol(path.actor, GetPatrol(path.patrol))
  end
end

function MoveSoldiersInside(soldier)
  AssignPatrol(soldier, GetPatrol("Patrol_49"))
  StartSequence()
  SequenceLockAI(soldier)
  Then()
  SequenceMoveIntoBuilding(soldier, GetLocation("Node_1"), MovementStyle.walking)
  EndSequence()
end

SoldiersAlerted = function()
  Globals.isArcheryDone = true
  UpdateScrollState(GetScroll("Scroll_1"), ScrollState.collected)
  AssignSoldiersToOfficer()
  AssignPatrolPaths()
end

RevealWeaponRoom = function()
  if not IsPatchApplied(GetPatchScript(WeaponRoom.patch)) then
    WeaponRoom:Reveal()
    ActivateActors({GetActor("Epee_Bleu_36")})
    AssignPatrol(GetActor("Epee_Bleu_36"), GetPatrol("Patrol_54"))
  end
end

RevealDungeonTower = function()
  if not IsPatchApplied(GetPatchScript(DungeonTower.patch)) then
    DungeonTower:Reveal()
    Activate(GetItem("Purses_5"))
    Activate(GetItem("Money_6"))

    ActivateActors({GetActor("Arbaletrier_Bleu_23"), GetActor("Worman")})
    AssignPatrol(GetActor("Worman"), GetPatrol("Patrol_25"))
  end
end

RevealBedRoom = function()
  if not IsPatchApplied(GetPatchScript(BedRoom.patch)) then
    BedRoom:Reveal()
    Activate(GetItem("Purses_10"))
    UpdateScrollState(GetScroll("Scroll_13"), ScrollState.collected)
  end
end

---Actors
PoorWeepingOne_80000344 = {
  ProcessMessage = function(message, unk2, unk3)
    if message == 1 then
      AssignPatrol(GetActor("Epee_Bleu_25"), GetPatrol("Patrol_26"))
      AssignPatrol(GetActor("Epee_Bleu_26"), GetPatrol("Patrol_27"))
      AssignPatrol(GetActor("Lancier_Bleu_27"), GetPatrol("Patrol_28"))
      AssignPatrol(GetActor("Lancier_Bleu_28"), GetPatrol("Patrol_29"))
      Globals.hasPoorWeepingOneRun = true
      StartSequence()
      SequenceMove(GetActor("Poor_Weeping_One"), GetLocation("Node_8"), MovementStyle.running)
      Then()
      SequenceTurnTo(GetActor("Poor_Weeping_One"), GetLocation("Node_9"))
      Then()
      SequenceCall(function()
        AttachScrollToNPC(GetActor("Poor_Weeping_One"), GetScroll("Scroll_15"))
      end)
      EndSequence()
    end
  end
}

Acher03_80000042 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

Acher02_80000041 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

Acher01_80000040 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end,

  ProcessMessage = function(message, unk2, unk3)
    if message == 1 and not Globals.isArcheryDone then
      StartSequence()
      SequenceSendMessage(GetActor("Archer_Officer"), 1)
      Then()
      SequenceTimer(12)
      Then()
      SequenceAction(GetActor("Shooting_Archer"), ActorAction.shoot, GetActorIndex(GetActor("Cible01_2")))
      Then()
      SequenceTimer(12)
      Then()
      SequenceSendMessage(GetActor("Archer_Officer"), 2)
      EndSequence()
    end
  end
}

SergentArchers_80000043 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end,

  ProcessMessage = function(message, unk2, unk3)
    if message == 1 and not Globals.isArcheryDone then
      StartSequence()
      SequenceTurnTo(GetActor("Archer_Officer"), GetLocation("Node_2"))
      EndSequence()
    end

    if message == 2 and not Globals.isArcheryDone then
      SetPersistentProperty(GetActor("Shooting_Archer"), PersistentProperty.arrows,
        (GetPersistentProperty(GetActor("Shooting_Archer"), PersistentProperty.arrows) + 1))
      StartSequence()
      SequenceTurnTo(GetActor("Archer_Officer"), GetActorLocation(GetActor("Shooting_Archer")))
      Then()
      SequencePlayAnim(GetActor("Archer_Officer"), AnimationAction.gatheringSoldiers)
      EndSequence()
    end
  end
}

SoldatCibles01_80000044 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

SoldatCibles02_8000004a = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

SoldatCibles03_80000050 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

LancierEscalierHcour01_8000005f = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

LancierHCour02_80000060 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      SoldiersAlerted()
    end
  end
}

BouclierXT01_800000d6 = {
  ActionChange = function(action, oldAction)
    if action == AnimationAction.alerted then
      AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_66"))
      AssignPatrol(GetActor("WomanCivilianPoor_2"), GetPatrol("Patrol_59"))
    end
  end
}

---Targets
Cible1_800001bb = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible00_1"))
  end
}

Cible2_800001bd = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible01_2"))
  end
}

Cible3_800001be = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible02_3"))
  end
}

Cible4_800001bf = {
  ActivatedByArrow = function(actorSource)
    HandleCibleHit(actorSource, GetActor("Cible03_4"))
  end
}

function HandleCibleHit(actorSource, cibleActor)
  if not Globals.isCibleHit and IsActorPC(actorSource) then
    Globals.isCibleHit = true
    StartSequence()
    SequencePlayAnimFreeze(cibleActor, AnimationAction.targetHit)
    Then()
    SequenceTimer(10)
    Then()
    SequenceCall(ArchersDone)
    EndSequence()
  end
end

mecanisme_pont_levis_800001e9 = {
  ActivatedBySword = function(actorSource)
    DrawBridgeRoom:ActivateDrawBridge(GetLocation("Node_15"))
	  Globals.lowered_bridge = true
  end
}

--WayPoints
OfficierdesArchers__6___80000184 = {
  ReachPoint = function(actor)
    StartSequence()
    SequenceCall(function() MoveSoldiersInside(GetActor("Archer_Officer")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Archer_Bleu_12")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Archer_Bleu_13")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Shooting_Archer")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Epee_Bleu_16")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Epee_Bleu_17")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Epee_Bleu_18")) end)
	SequenceTimer(12)
	Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Soldier_53")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Soldier_54")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Soldier_57")) end)
    SequenceTimer(12)
    Then()
    SequenceCall(function() MoveSoldiersInside(GetActor("Soldier_58")) end)
    EndSequence()
  end
}

GirlWay01__1___800001f8 = {
  ReachPoint = function(actor)
    SetPatrolShouldRun(GetActor("Flirting_Swordsman"), true)
    AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_61"))
    AssignPatrol(actor, GetPatrol("Patrol_65"))
  end
}

GirlWay02__1___800001f9 = {
  ReachPoint = function(actor)
    SetPatrolShouldRun(GetActor("Flirting_Swordsman"), false)
    AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_60"))
    AssignPatrol(actor, GetPatrol("Patrol_64"))
    StareActor(GetActor("Flirting_Swordsman"), GetActorScript(MiscAnimation.riverPier))
  end
}

--Scolls
ParchArcherDebut_80000258 = {
  Taken = function(actorSource)
    if not Globals.isArcheryDone then
      AddObjective(5, false)
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetActorLocation(GetActor("Archer_Officer")))
      Then()
      SequenceTimer(15)
      Then()
      SequenceSpeak(GetActor("Archer_Officer"), 61)
      Then()
      SequenceDisplayPopupText(5)
      Then()
      SequenceDisplayPopupText(22)
      Then()
      SequenceSendMessage(GetActor("Shooting_Archer"), 1)
      Then()
      SequenceFreezeAll(false)
      SequenceUnLockUser()
      EndSequence()
    end
  end
}

ParchArgent_8000047f = {
  Taken = function(actorSource)
    if Globals.isArcheryDone then
      DisplayPopupText(19)
      Activate(GetItem("Money_9"))
      SetScrollState(ThisScroll(), ScrollState.collected)
    end
  end
}

ParchEdwardSpeak_8000025a = {
  Taken = function(actorSource)
    AddObjective(1, true)
    CompleteObjective(0)
    Reveal(GetActor("ManCivilianFriend_5"))

	  StartSequence()
    SequenceSetZoomLevel(ZoomLevel.default)
    Then()
    SequenceLockUser()
   -- SequenceFreezeAll(true)
    Then()
    SequenceDisplayPopupText(8)
    Then()
    SequenceDisplayPopupText(9)
    Then()
    SequenceDisplayPopupText(10)
    Then()
    SequenceScrollCameraTo(GetLocation("Node_10"))
    Then()
    SequenceMove(GetActor("ManCivilianFriend_5"), GetLocation("Node_10"), MovementStyle.walking)
    Then()
    SequenceCall(function()
      AssignPost(GetActor("ManCivilianFriend_5"), GetLocation("Node_10"), ActorDirection.south)
      AttachScrollToNPC(GetActor("ManCivilianFriend_5"), GetScroll("Scroll_4"))
    end)
    Then()
    SequenceTimer(40)
    Then()
    SequenceCall(call_reinforce) -- function that call reinforcements
    SequenceJumpCameraTo(GetActorLocation(GetRobin()))
    SequenceFreezeAll(false)
    SequenceUnLockUser()
    EndSequence()

    RemoveScrollFromNPC(GetActor("Edward"))
    SetScrollState(ThisScroll(), ScrollState.collected)
  end
}

ParchFilsEdward_8000025b = {
  Taken = function(actorSource)
    AddObjective(2, true)
    CompleteObjective(1)
    DisplayPopupText(11)
    Globals.victory = true
    RemoveScrollFromNPC(GetActor("ManCivilianFriend_5"))
    SetScrollState(ThisScroll(), ScrollState.collected)
  end
}

TutGrimper_8000025d = {
  Taken = function(actorSource)
    StartSequence()
    SequenceSetZoomLevel(ZoomLevel.default)
    Then()
    SequenceLockUser()
    Then()
    SequenceFreezeAll(true)
    Then()
    SequenceScrollCameraTo(GetLocation("Node_12"))
    Then()
    SequenceTimer(12)
    Then()
    SequenceDisplayPopupText(12)
    Then()
    SequenceTimer(12)
    Then()
    SequenceFreezeAll(false)
    SequenceUnLockUser()
    EndSequence()
  end
}

TutJump_8000025e = {
  Taken = function(actorSource)
    DisplayPopupText(13)
  end
}

TutKO_8000025f = {
  Taken = function(actorSource)
    DisplayPopupText(14)
    DisplayPopupText(21)
  end
}

TutBeggar_80000261 = {
  Taken = function(actorSource)
    StartSequence()
    SequenceSetZoomLevel(1.0)
    Then()
    SequenceLockUser()
    Then()
    SequenceScrollCameraTo(GetLocation("Node_1"))
    Then()
    SequenceTimer(15)
    Then()
    SequenceReveal(GetActor("Mendicant_6"))
    Then()
    SequenceDisplayPopupText(16)
    Then()
    SequenceTimer(40)
    SequenceJumpCameraTo(GetActorLocation(actorSource))
    Then()
    SequenceUnLockUser()
    EndSequence()
  end
}

TutGetObject_80000262 = {
  Taken = function(actorSource)
    DisplayPopupText(15)
    if Globals.TutorialGetObjectFirst then
      Globals.TutorialGetObjectFirst = false
      DisplayPopupText(17)
      Activate(GetItem("Purses_7"))
    end
  end
}

BeggarWorman_80000277 = {
  Taken = function(actorSource)
    if not IsBonusItemPickedUp(GetItem("Money_6")) then
      AddObjective(3, false)
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetLocation("Node_4"))
      Then()
      SequenceCall(RevealDungeonTower)
      Then()
      SequenceReveal(GetActor("Worman"))
      SequenceReveal(GetActor("Arbaletrier_Bleu_23"))
      Then()
      SequenceTimer(15)
      Then()
      SequenceDisplayPopupText(4)
      Then()
      SequenceTimer(40)
      Then()
      SequenceJumpCameraTo(GetActorLocation(actorSource))
      Then()
      SequenceUnLockUser()
      SequenceFreezeAll(false)
      EndSequence()
      SetScrollState(ThisScroll(), ScrollState.collected)
    end
  end
}

BeggarHaldric_80000278 = {
  Taken = function(actorSource)
    if GetPersistentProperty(GetActor("Haldric"), PersistentProperty.money) > 0 then
      AddObjective(4, false)
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceDisplayPopupText(7)
      Then()
      SequenceScrollCameraTo(GetActorLocation(GetActor("Haldric")))
      Then()
      SequenceTimer(15)
      Then()
      SequenceReveal(GetActor("Haldric"))
      Then()
      SequenceTimer(40)
      Then()
      SequenceJumpCameraTo(GetActorLocation(actorSource))
      Then()
      SequenceFreezeAll(false)
      SequenceUnLockUser()
      EndSequence()
      SetScrollState(ThisScroll(), ScrollState.collected)
    end
  end
}

Beggarpatch01_800002c4 = {
  Taken = function(actorSource)
    DrawBridgeRoom:Reveal()
    StartSequence()
    SequenceSetZoomLevel(ZoomLevel.default)
    Then()
    SequenceLockUser()
    SequenceFreezeAll(true)
    Then()
    SequenceScrollCameraTo(GetLocation("Node_14"))
    Then()
    SequenceTimer(15)
    Then()
    SequenceDisplayPopupText(20)
    Then()
    SequenceTimer(40)
    Then()
    SequenceJumpCameraTo(GetActorLocation(actorSource))
    Then()
    SequenceFreezeAll(false)
    SequenceUnLockUser()
    EndSequence()
  end
}

TutOpenSalle_8000046d = {
  Taken = function(actorSource)
    if not IsPatchApplied(GetPatchScript(7)) then
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetLocation("Node_13"))
      Then()
      SequenceTimer(15)
      Then()
      SequenceCall(RevealBedRoom)
      Then()
      SequenceTimer(40)
      Then()
      SequenceJumpCameraTo(GetActorLocation(actorSource))
      Then()
      SequenceFreezeAll(false)
      SequenceUnLockUser()
      EndSequence()
    end
  end
}

ArrowsAtEnd_800006e3 = {
  Taken = function(actorSource)
    Activate(GetItem("Arrows_3"))
    SetScrollState(ThisScroll(), ScrollState.collected)
  end
}

PoorManScroll_80000349 = {
  Taken = function(actorSource)
    Activate(GetItem("Clovers_11"))
    RemoveScrollFromNPC(GetActor("Poor_Weeping_One"))
    SetScrollState(ThisScroll(), ScrollState.collected)
    StartSequence()
    SequenceSpeak(GetActor("Poor_Weeping_One"), 86)
    Then()
    SequenceDisplayPopupText(3)
    EndSequence()
  end
}


---Zones
ZP06Sud_80000170 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealWeaponRoom()
    end
  end
}

ZP06Est_80000171 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealWeaponRoom()
    end
  end
}

ZP03TourOuest_80000173 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealDungeonTower()
    end
  end
}

ZP01TourPtLevis_80000175 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if not IsPatchApplied(GetPatchScript(DrawBridgeRoom.patch)) then
        DrawBridgeRoom:Reveal()
        SetScrollState(GetScroll("Scroll_12"), ScrollState.collected)
      end
    end
  end
}

ZP04CentralBas_80000177 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      Battlement:Reveal()
    end
  end
}

ZP07Bas_80000178 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealBedRoom()
    end
  end
}

ZP07Haut_80000179 = {
  Enter = function(actor)
    if IsActorPC(actor) then
      RevealBedRoom()
    end
  end
}

ZP05GdSalle_8000017a = {
  Enter = function(actor)
    if IsActorPC(actor) then
      if not IsPatchApplied(GetPatchScript(StairCase.patch)) then
        StairCase:Reveal()
      end
      if not IsPatchApplied(GetPatchScript(ThroneRoom.patch)) then
        ThroneRoom:Reveal()

        Activate(GetActor("Edward"))
        AttachScrollToNPC(GetActor("Edward"), GetScroll("Scroll_3"))
        Reveal(GetActor("Edward"))
        UnlockAI(GetActor("Edward"))
      end
    end
  end
}

ZP09EscalierBas_8000017b = {
  Enter = function(actor)
    if IsActorPC(actor) and not IsPatchApplied(GetPatchScript(StairCase.patch)) then
      StairCase:Reveal()
    end
  end
}

ZP09EscalierHaut_8000017c = {
  Enter = function(actor)
    if IsActorPC(actor) and not IsPatchApplied(GetPatchScript(StairCase.patch)) then
      StairCase:Reveal()
    end
  end
}

EdwardZone_80000275 = {
  Enter = function(actor)
    if IsActorPC(actor) and Globals.EdwardZoneFirst then
      StopActor(actor)
      Globals.EdwardZoneFirst = false
      StartSequence()
      SequenceSetZoomLevel(ZoomLevel.default)
      Then()
      SequenceLockUser()
      SequenceFreezeAll(true)
      Then()
      SequenceScrollCameraTo(GetLocation("Node_5"))
      Then()
      SequenceTimer(15)
      Then()
      SequenceDisplayPopupText(18)
      Then()
      SequenceTimer(40)
      Then()
      SequenceUnLockUser()
      SequenceFreezeAll(false)
      EndSequence()
    end
  end
}

persecutionZone_80000346 = {
  Exit = function(actor)
    if GetNumberOfActorsInSector(GetLocation("Node_27")) == 1 then
      if not Globals.hasPoorWeepingOneRun then
        SendMessage(GetActor("Poor_Weeping_One"), 1)
      end
    end
  end
}

-- MY ADDS


function call_reinforce() -- Activate and unlock reinforce
	
	local reinforcements = {
    "Reinforce_off",
    "Reinforce_1",
    "Reinforce_2",
    "Reinforce_3",
    "Reinforce_4",
	  "taxman",
    "Civilian_9",
    "Reinforce_5",
    "Reinforce_6",
    "Reinforce_7",
    "Reinforce_8"
	}

  for _, actor in pairs(reinforcements) do
    SetActorLocation(GetActor(actor), NoWhere())
    Activate(GetActor(actor))
    --Reveal(GetActor(actor))
  end

  local timestep = 30

	StartSequence()
	SequenceUnlockAI(GetActor("Reinforce_off"))
	Then()
	SequenceEnterLevel(GetActor("Reinforce_off"), GetLocation("Node_30"), 7, MovementStyle.walking)
  EndSequence()
  
	StartSequence()
  SequenceTimer(timestep)
  Then()
	SequenceEnterLevel(GetActor("Reinforce_1"), GetLocation("Node_31"), 7, MovementStyle.walking)
	SequenceEnterLevel(GetActor("Reinforce_2"), GetLocation("Node_32"), 7, MovementStyle.walking)
	Then()
	SequenceUnlockAI(GetActor("Reinforce_1"))
	SequenceUnlockAI(GetActor("Reinforce_2"))
  EndSequence()
  
	StartSequence()
  SequenceTimer(2 * timestep)
  Then()
	SequenceEnterLevel(GetActor("Reinforce_3"), GetLocation("Node_33"), 7, MovementStyle.walking)
	SequenceEnterLevel(GetActor("Reinforce_4"), GetLocation("Node_34"), 7, MovementStyle.walking)
	Then()
	SequenceUnlockAI(GetActor("Reinforce_3"))
	SequenceUnlockAI(GetActor("Reinforce_4"))
  EndSequence()
  
	StartSequence()
  SequenceTimer(3 * timestep)
  Then()
	SequenceEnterLevel(GetActor("taxman"), GetLocation("Node_35"), 7, MovementStyle.walking)
	SequenceEnterLevel(GetActor("Civilian_9"), GetLocation("Node_36"), 7, MovementStyle.walking)
	Then()
	SequenceUnlockAI(GetActor("Civilian_9"))
	SequenceUnlockAI(GetActor("taxman"))
  EndSequence()
  
	StartSequence()
  SequenceTimer(4 * timestep)
  Then()
	SequenceEnterLevel(GetActor("Reinforce_5"), GetLocation("Node_37"), 7, MovementStyle.walking)
	SequenceEnterLevel(GetActor("Reinforce_6"), GetLocation("Node_38"), 7, MovementStyle.walking)
	Then()
	SequenceUnlockAI(GetActor("Reinforce_6"))
	SequenceUnlockAI(GetActor("Reinforce_5"))
  EndSequence()
  
	StartSequence()
  SequenceTimer(5 * timestep)
  Then()
	SequenceEnterLevel(GetActor("Reinforce_7"), GetLocation("Node_39"), 7, MovementStyle.walking)
	SequenceEnterLevel(GetActor("Reinforce_8"), GetLocation("Node_40"), 7, MovementStyle.walking)
	Then()
	SequenceUnlockAI(GetActor("Reinforce_7"))
	SequenceUnlockAI(GetActor("Reinforce_8"))
	
	SequenceCall(function()
    ReinforceToOfficer()	-- assign group
    AssignPatrol(GetActor("Reinforce_off"), GetPatrol("Patrol_74")) -- assign path
	end)
  EndSequence()
end

function ReinforceToOfficer() -- function that assign units Reinforce to Reinforce Officer 
  local subordinates = {
    GetActor("Reinforce_1"),
    GetActor("Reinforce_2"),
	  GetActor("Reinforce_3"),
    GetActor("Reinforce_4"),
	  GetActor("Reinforce_5"),
    GetActor("Reinforce_6"),
	  GetActor("Reinforce_7"),
    GetActor("Reinforce_8"),
	  GetActor("taxman"),
    GetActor("Civilian_9")
  }

  for _, subordinate in pairs(subordinates) do
    AddAsSquadMember(GetActor("Reinforce_off"), subordinate)
  end
end


Reinforce_script_1={ -- drawbridge opening, add soldier animation opening bridge
	ReachPoint = function(actor)
		local reinforcements_freeze = {
			"Hallebardier_Bleu_3",
			"Reinforce_off",
			"Reinforce_1",
			"Reinforce_2",
			"Reinforce_3",
			"Reinforce_4",
			"Reinforce_5",
			"Reinforce_6",
			"Reinforce_7",
			"Reinforce_8",
			"taxman",
			"Civilian_9",
			"Soldier_49",
			"Soldier_51"
			}
		
		AssignPatrol(GetActor("Reinforce_off"), nil)
		RemoveAllSquadMembers(GetActor("Reinforce_off"))
		
		AssignPatrol(GetActor("Reinforce_1"), GetPatrol("Patrol_103"))
		AssignPatrol(GetActor("Reinforce_2"), GetPatrol("Patrol_104"))
		AssignPatrol(GetActor("Reinforce_3"), GetPatrol("Patrol_105"))
		AssignPatrol(GetActor("Reinforce_4"), GetPatrol("Patrol_106"))
		AssignPatrol(GetActor("Reinforce_5"), GetPatrol("Patrol_109"))
		AssignPatrol(GetActor("Reinforce_6"), GetPatrol("Patrol_110"))
		AssignPatrol(GetActor("Reinforce_7"), GetPatrol("Patrol_111"))
		AssignPatrol(GetActor("Reinforce_8"), GetPatrol("Patrol_112"))
		AssignPatrol(GetActor("taxman"), GetPatrol("Patrol_107"))
		AssignPatrol(GetActor("Civilian_9"), GetPatrol("Patrol_108"))
		
		SetAIState(GetActor("Reinforce_off"), AiState.script)
		SetAIState(GetActor("Hallebardier_Bleu_3"), AiState.script)
		
		StartSequence()
		
		SequenceTimer(50)
		
		if not Globals.lowered_bridge then
			Globals.lowered_bridge = true
			Then()
			SequenceSpeak(GetActor("Reinforce_off"), 75)
			Then()
			SequencePlayAnim(GetActor("Reinforce_off"), AnimationAction.gatheringSoldiers)
			Then()
			SequenceCall(function()
				AssignPatrol(GetActor("Hallebardier_Bleu_3"), nil)
			end)
			Then()
			SequenceLockAI(GetActor("Hallebardier_Bleu_3"))
			Then()
			if not IsPatchApplied(GetPatchScript(DrawBridgeRoom.patch)) then
				SequenceMoveIntoBuilding(GetActor("Hallebardier_Bleu_3"), GetLocation("Node_28"), MovementStyle.running)
				Then()
				SequenceTimer(200)
			else
				--SequenceCall(function()
				--	AssignPatrol(GetActor("Hallebardier_Bleu_3"), GetPatrol("Patrol_75"))
				--	SetPatrolShouldRun(GetActor("Hallebardier_Bleu_3"), true)
				--end)
				SequenceMove(GetActor("Hallebardier_Bleu_3"), GetLocation("Node_14"), MovementStyle.running)
				Then()
				SequenceTimer(210)
				Then()
				SequenceTurnTo(GetActor("Hallebardier_Bleu_3"), GetLocation("Node_3"))
				Then()
				SequenceAction(GetActor("Hallebardier_Bleu_3"), ActorAction.strikeLeftToRight, GetActorIndex(GetActor("Treadwheel")))
				Then()
				SequenceTimer(10)
			end
			Then()
			for _, actor in pairs(reinforcements_freeze) do
				SequenceLockAI(GetActor(actor)) 
				Then()
			end
			
			Then()
			SequenceCall(function()
				DrawBridgeRoom:ActivateDrawBridge(GetLocation("Node_15"))
			end)
				
			Then()
			SequenceTimer(40)
			Then()
			
			for _, actor in pairs(reinforcements_freeze) do
				SequenceUnlockAI(GetActor(actor))
				Then()
			end
			SequenceTimer(10)
			
		end
		Then()
		SequenceCall(function()
			AssignPatrol(GetActor("Hallebardier_Bleu_3"), GetPatrol("Patrol_6"))
			SetPatrolShouldRun(GetActor("Hallebardier_Bleu_3"), false)
			ReinforceToOfficer()
			AssignPatrol(GetActor("Reinforce_off"), GetPatrol("Patrol_76")) 
		end)
		
		EndSequence()
		
	end	
}

Reinforce_script_2={
	ReachPoint = function(actor)
		AssignPatrol(GetActor("Reinforce_off"), nil) 
		RevealDungeonTower()
		Battlement:Reveal()
		
		StartSequence()
		SequenceTurnTo(GetActor("Reinforce_off"), GetLocation("Node_29"))
		Then()
		SequenceTimer(40)
		Then()
		SetAIState(GetActor("Reinforce_off"), AiState.script)
		Then()
		SequenceSpeak(GetActor("Reinforce_off"), 64)  
		Then()
		SequencePlayAnim(GetActor("Reinforce_off"), AnimationAction.gatheringSoldiers)
		Then()
		SequenceTimer(10)	
		Then()
		SequenceCall(function()
			RemoveAllSquadMembers(GetActor("Reinforce_off"))
				
			AssignPatrol(GetActor("Reinforce_off"), GetPatrol("Patrol_82"))		
			AssignPatrol(GetActor("Reinforce_1"), GetPatrol("Patrol_78"))		
			AssignPatrol(GetActor("Reinforce_2"), GetPatrol("Patrol_80"))		
			AssignPatrol(GetActor("Reinforce_3"), GetPatrol("Patrol_77"))		
			AssignPatrol(GetActor("Reinforce_4"), GetPatrol("Patrol_79"))		
			AssignPatrol(GetActor("Reinforce_8"), GetPatrol("Patrol_81"))		
			AssignPatrol(GetActor("Reinforce_7"), GetPatrol("Patrol_83"))			
			AssignPatrol(GetActor("Reinforce_5"), GetPatrol("Patrol_84"))
			SequenceTimer(50)			
			AssignPatrol(GetActor("Reinforce_6"), GetPatrol("Patrol_84"))
			SequenceTimer(30)	
			AssignPatrol(GetActor("taxman"), GetPatrol("Patrol_84"))
			SequenceTimer(30)	
			AssignPatrol(GetActor("Civilian_9"), GetPatrol("Patrol_84"))			
		end)
		EndSequence()
	end
}

Reinforce_script_3 = {
    ReachPoint = function(actor)
        local actorName = GetActorName(actor)
        if actorName == "Reinforce_5" then
            AssignPatrol(actor, GetPatrol("Patrol_85"))
        elseif actorName == "Reinforce_6" then
            AssignPatrol(actor, GetPatrol("Patrol_87"))
        elseif actorName == "taxman" then
            AssignPatrol(actor, GetPatrol("Patrol_86"))
        elseif actorName == "Civilian_9" then
            AssignPatrol(actor, GetPatrol("Patrol_88"))
        end
    end
}

sitting_1= {
	ReachPoint = function(actor)
		SetAIState(actor, AiState.script) 
		StartSequence()
		SequenceCall(function()
			SetActorPosture(actor, 16)
		end) 
		EndSequence()
	end
}

sitting_2= {
	ReachPoint = function(actor)
		SetAIState(actor, AiState.script) 
		StartSequence()
		SequenceCall(function()
			SetActorPosture(actor, 16)
		end) 
		EndSequence()
	end
}

special_1= {
	ReachPoint = function(actor)
		SetAIState(actor, AiState.script) 
		StartSequence()
		SequencePlayAnim(actor, 269) 
		Then() 
		SequencePlayAnimLoop(actor, 270) 
		EndSequence()
	end
}

special_2= {
	ReachPoint = function(actor)
		SetAIState(actor, AiState.script) 
		StartSequence()
		SequencePlayAnim(actor, 269) 
		Then() 
		SequencePlayAnimLoop(actor, 270) 
		EndSequence()
	end
}

Reinforce_move_over_1_Start= {
	ReachPoint = function(actor)
		AssignPatrol(GetActor("Lancier_Bleu_33"), GetPatrol("Patrol_90"))	
	end
}	
		
Reinforce_move_over_1_End= {
	ReachPoint = function(actor)		
		AssignPatrol(GetActor("Lancier_Bleu_33"), GetPatrol("Patrol_91"))
    end
}

Reinforce_move_over_2_Start= {
	ReachPoint = function(actor)
		AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_92"))
		AssignPatrol(GetActor("WomanCivilianPoor_2"), GetPatrol("Patrol_93"))
    end
}

Reinforce_move_over_3_Start_2_End = {
	ReachPoint = function(actor)
		AssignPatrol(GetActor("Lancier_Bleu_30"), GetPatrol("Patrol_94"))
		AssignPatrol(GetActor("Flirting_Swordsman"), GetPatrol("Patrol_59"))
		AssignPatrol(GetActor("WomanCivilianPoor_2"), GetPatrol("Patrol_64"))
	end
}			
			
Reinforce_move_over_3_End= {
	ReachPoint = function(actor)		
		AssignPatrol(GetActor("Lancier_Bleu_30"), GetPatrol("Patrol_32"))
    end
}

Reinforce_move_over_4_Start= {
	ReachPoint = function(actor)
			AssignPatrol(GetActor("Hallebardier_Bleu_11"), GetPatrol("Patrol_13"))
	end
}			
Reinforce_move_over_4_End= {
	ReachPoint = function(actor)		
		AssignPatrol(GetActor("Hallebardier_Bleu_11"), GetPatrol("Patrol_10"))
    end
}

Reinforce_move_over_5_Start= {
	ReachPoint = function(actor)
		AssignPatrol(GetActor("Hallebardier_Bleu_10"), GetPatrol("Patrol_95"))
   end
}
				
Reinforce_move_over_6_Start_5_End= {
	ReachPoint = function(actor)
		AssignPatrol(GetActor("Epee_Bleu_36"), GetPatrol("Patrol_96"))
		AssignPatrol(GetActor("Hallebardier_Bleu_10"), GetPatrol("Patrol_4"))
    end
}	
	
Reinforce_move_over_6_End= {
	ReachPoint = function(actor)		
		AssignPatrol(GetActor("Epee_Bleu_36"), GetPatrol("Patrol_54"))
    end
}

Move_Arbaletrier= {
	ReachPoint = function(actor)		
		AssignPatrol(GetActor("Arbaletrier_Bleu_23"), GetPatrol("Patrol_100"))
    end
}

Script_Reinforce_off={
	ActionChange = function(action, oldAction)
		if action == AnimationAction.alerted then
			RemoveAllSquadMembers(GetActor("Reinforce_off"))
			SetAlwaysAttentive(GetActor("Reinforce_off"), 1)
			SetAlwaysAttentive(GetActor("Reinforce_1"), 1)
			SetAlwaysAttentive(GetActor("Reinforce_2"), 1)
			SetAlwaysAttentive(GetActor("Reinforce_3"), 1)
			SetAlwaysAttentive(GetActor("Reinforce_4"), 1)
			SetAlwaysAttentive(GetActor("Reinforce_5"), 1)
			SetAlwaysAttentive(GetActor("Reinforce_6"), 1)
			SetAlwaysAttentive(GetActor("Reinforce_7"), 1)
			SetAlwaysAttentive(GetActor("Reinforce_8"), 1)
			
			if not Globals.lowered_bridge then
				AssignPatrol(GetActor("Reinforce_off"), GetPatrol("Patrol_121"))
				AssignPatrol(GetActor("Reinforce_1"), GetPatrol("Patrol_117"))
				AssignPatrol(GetActor("Reinforce_2"), GetPatrol("Patrol_118"))
				AssignPatrol(GetActor("Reinforce_3"), GetPatrol("Patrol_119"))
				AssignPatrol(GetActor("Reinforce_4"), GetPatrol("Patrol_120"))
				AssignPatrol(GetActor("Reinforce_5"), GetPatrol("Patrol_123"))
				AssignPatrol(GetActor("Reinforce_6"), GetPatrol("Patrol_124"))
				AssignPatrol(GetActor("Reinforce_7"), GetPatrol("Patrol_125"))
				AssignPatrol(GetActor("Reinforce_8"), GetPatrol("Patrol_126"))
				AssignPatrol(GetActor("taxman"), GetPatrol("Patrol_127"))
				AssignPatrol(GetActor("Civilian_9"), GetPatrol("Patrol_122"))
			else
				AssignPatrol(GetActor("Reinforce_off"), GetPatrol("Patrol_135"))
				AssignPatrol(GetActor("Reinforce_1"), GetPatrol("Patrol_129"))
				AssignPatrol(GetActor("Reinforce_2"), GetPatrol("Patrol_130"))
				AssignPatrol(GetActor("Reinforce_3"), GetPatrol("Patrol_131"))
				AssignPatrol(GetActor("Reinforce_4"), GetPatrol("Patrol_132"))
				AssignPatrol(GetActor("Reinforce_5"), GetPatrol("Patrol_137"))
				AssignPatrol(GetActor("Reinforce_6"), GetPatrol("Patrol_136"))
				AssignPatrol(GetActor("Reinforce_7"), GetPatrol("Patrol_138"))
				AssignPatrol(GetActor("Reinforce_8"), GetPatrol("Patrol_139"))
				AssignPatrol(GetActor("taxman"), GetPatrol("Patrol_134"))
				AssignPatrol(GetActor("Civilian_9"), GetPatrol("Patrol_133"))
			end
		end	
	end	
}

function Combact_soldier_1()
	local actor = GetActor("Soldier_53")
	SetAIState(actor, 7) 
	StartSequence()
	Then()
	SequencePlayAnim(actor, AnimationAction.raisingShield)
	Then()
	SequencePlayAnim(actor, AnimationAction.waitingShield)
    Then()
	for i = 1, 7 do
	SequencePlayAnim(actor, AnimationAction.waitingShield)
	Then()
	end
	SequencePlayAnim(actor, AnimationAction.repelArrow)
	Then()
	SequencePlayAnim(actor, AnimationAction.loweringShield)
	EndSequence()
  end
  
  function Combact_soldier_2()
	local actor = GetActor("Soldier_54")
	SetAIState(actor, 7) 
	StartSequence()
	Then()
	SequenceAction(actor, ActorAction.shoot, GetActorIndex(GetActor("Soldier_53")))
	EndSequence()
  end

--TEST

--Test= {												
--  ReachPoint = function(actor)
		
		
		--local actor1 = GetActor("Hallebardier_Bleu_8")
	
--		SetAIState(actor, 7)  

--		StartSequence()
		
--		Then()
		--SequencePlayAnimLoop(actor1, 297)
		--SequencePlayAnim(actor1, 6)
	
--		EndSequence()

--	end
--}
